/*
*********************************************************************************************************
*
*                                        BOARD SUPPORT PACKAGE
*
*                                             Renesas M16C
*                                                on the
*                                      RSKM16C62P Evaluation Board + FAT Board
*
*                                      Keypad scan routines for Renesas M16C62P
*                                           on the FAT base board 
*
* Filename      : keyscan_4x4.c
* Version       : V1.00
* Programmer(s) : Ralf Patz (based on Brian Nagel's work)
*********************************************************************************************************
*/

#include <includes.h>

/*
*********************************************************************************************************
*                                            GLOBAL VARIABLES
*********************************************************************************************************
*/
CPU_INT08U debouncing;
typedef struct{
                CPU_INT16U data:8;    // holds the data associated with button
                CPU_INT16U unread:1;  // indicates when new button pressed 
                CPU_INT16U live:1;    // indicates that button has been pressed  
                                        // and is still pressed
              } key_t;
              
key_t button={0,0,0};


/*
*********************************************************************************************************
*                                           KEYSCAN INITIALIZATION
*
* Description : This function should be called by your application code before you make use of any of the
*               functions found in this module. It can only be called AFTER the OSStart() function.
*
* Arguments   : none
*
* Returns     : none
*********************************************************************************************************
*/
void keyscan_4x4_init(void)
{
	CPU_INT16U i,j;
    CPU_INT08U err;         // Error variable

    disable_interrupt();    //disable interrupts

	P8D_7=0;                //input for Fc osc
	P8D_6=0;                //input for Fc osc
	PRCR_0=1;               //protection mode disabled
	CM0_4=1;                //start Fc OSC
	CM0_7=0;                //cpu clock main 

	for (i=0; i<2047; i++)
	{
	 	for (j=0; j<2047; j++);        //allow time for oscillator to stabilise
	
   	}

	CM0_3 = 0;		        // Set Xc clock to low drive
	PRCR_0=0;               // enable protection

	P10D=0x0f;              //lower nibble as output upper nibble as input
	PUR2=0x20;              //pull up resistors on upper nibble
	KUPIC=0x05;             //enable key input interrupt with priority level of 5
	TA0MR=0xc2;             //enable timer 0 as one shot timer with Fc as clock source
	TA0=1;                  //give reload register a non zero value
	TA0IC=0x04;             //set priority level
	enable_interrupt();     //enable interrupts
	TABSR_0=1;              //start timer
    
}

/*
*********************************************************************************************************
*                                           GET KEY
*
* Description : This function retrieves the press key.
*
* Arguments   : none
*
* Returns     : ASCII coded character for the pressed key
*               'X' if no key was pressed
*********************************************************************************************************
*/  
CPU_INT08U getcommand(void)
{
    CPU_INT08U command;
    
    disable_interrupt();
    command=button.unread?button.data:'X';    //if button unread then data ready to read otherwise no data return 'X'
    button.unread=0;
    enable_interrupt();
    return(command);
}
  
/*
*********************************************************************************************************
*                                           TIMER A0 ISR HANDLER
*
* Description : This function is called by the ISR (TA0ISR())
*
* Arguments   : none
*
* Returns     : none
*********************************************************************************************************
*/
//void Timer_A0(void) 

void Timer_A0(void) 
{
    static CPU_INT08U pattern;
    

    button.live=1;
    if (debouncing)       //if first push of button scan key
    {
        debouncing=0;
        TA0=2000;           //give around 2 secs before causing routine to rerun --delay before auto repeat 
        P10=0x0E;
        if ((pattern=P10)==0xee)
            button.data='1';
        else if (pattern==0xde)
            button.data='4';
        else if (pattern==0xbe)
            button.data='7';
        else if (pattern==0x7e)
            button.data='*';
        else
        {
            P10=0x0d;
            if ((pattern=P10)==0xed)
                button.data='2';
            else if (pattern==0xdd)
                button.data='5';
            else if (pattern==0xbd)
                button.data='8';
            else if (pattern==0x7d)
                button.data='0';
            else
            {
                P10=0x0b;
                if ((pattern=P10)==0xeb)
                    button.data='3';
                else if (pattern==0xdb)
                    button.data='6';
                else if (pattern==0xbb)
                    button.data='9';
                else if (pattern==0x7b)
                    button.data='#';
                else
                {
                    P10=0x07;
                    if ((pattern=P10)==0xe7)
                        button.data='A';
                    else if (pattern==0xd7)
                        button.data='B';
                    else if (pattern==0xb7)
                        button.data='C';
                    else if (pattern==0x77)
                        button.data='D';
                    else
                        button.live=0;
                }
            }
        }
    }
    else
    {
        P10=pattern;          //assign P10 with pattern of previous scan
        if (P10==pattern)     //if P10 is still the same as pattern
            TA0=204;           //give small delay be rerun isr --delay between auto repeat
        else
        {
            button.live=0;    //button no longer held
            ONSF=0;
        }
    }
   
    P10=0x00;              //clear port
    KUPIC_3=0;             //clear iterrupt request flag

    if (button.live)        //if button live, set unread flag to 1 and start count down
    {
        button.unread=1;
        ONSF=0x01;
    }
    
}

/*
*********************************************************************************************************
*                                           KEY INPUT ISR HANDLER
*
* Description : This function is called by the ISR (KeybISR())
*
* Arguments   : none
*
* Returns     : none
*********************************************************************************************************
*/

void KYB(void)  
{
    TA0=25;                 //when Keypad causes interrupt load TA0 with debounce delay
    debouncing=1;           //indicate new button pushed needs debouncing
    ONSF=0x01;              //start one shot timer and scan keypad after delay
}
 ISR HANDLER
*
* Description : This function is called by the ISR (KeybISR())
*
* Arguments   : none
*
* Returns     : none
*******************************************************************************************************